"""
This script computes anharmonic properties and thermal conductivity
for silicon bulk using hiPhive and the Boltzmann Transport Equation (BTE).

Usage:
    python TC.py <c_parameter>
"""

import sys
import numpy as np
import pickle
from kaldo.conductivity import Conductivity
from kaldo.forceconstants import ForceConstants
from kaldo.phonons import Phonons


def main():
    """
    Main function to compute phonon properties and thermal conductivity.
    It reads the parameter c from command line arguments and performs calculations.
    """
    # Get parameter c from command line arguments
    c = float(sys.argv[1])

    # Define initial supercell
    ini_sc = [7, 7, 7]
    root = "../../0_DFT/" + "c_{}/fc_kaldo_sc{}/".format(c, ini_sc[0])

    # Create force constants used for phonon calculations.
    # Only second order force constants are used.
    forceconstants = ForceConstants.from_folder(
        root,
        supercell=ini_sc,
        third_supercell=np.array([5, 5, 5]),
        format="shengbte-d3q",
        only_second=True,
    )

    # Compute phonon properties on a k-point mesh for a given temperature.
    for k_points in [12]:
        for temperature in [300]:
            phonons_config = {
                "kpts": [k_points, k_points, k_points],
                "is_classic": False,
                "temperature": temperature,
                "folder": "ald_c_{}_sc{}".format(c, ini_sc[0]),
                "storage": "numpy",
            }
            phonons = Phonons(forceconstants=forceconstants, **phonons_config)
            print("Masses: ", phonons.atoms.get_masses())

            # Adjust bandwidth using Lanczos interpolation
            # freq = phonons.frequency
            _ = phonons.heat_capacity  # This call creates necessary folders.

            root_bandwidth = "ald_c_{}_sc7/{}_{}_{}/{}/quantum/bandwidth.npy".format(
                c, k_points, k_points, k_points, temperature
            )
            pwd = "../"
            bw = np.load(pwd + root_bandwidth)

            with open("../../bw/interpolator_lanczos_c{}.pkl".format(c), "rb") as f:
                spl = pickle.load(f)

            bw_tot = bw + spl(phonons.frequency.flatten()).reshape(bw.shape)
            np.save(root_bandwidth, bw_tot)
            print("\n")

            # Compute thermal conductivity using Relaxation Time Approximation (RTA)
            rta_cond_matrix = Conductivity(
                phonons=phonons, method="rta", storage="numpy"
            ).conductivity.sum(axis=0)
            print(
                "Conductivity from RTA (W/m-K): %.3f"
                % (np.mean(np.diag(rta_cond_matrix)))
            )
            print(rta_cond_matrix)
            print("\n")

            # Compute thermal conductivity using QHGK method
            qhgk_cond_matrix = Conductivity(
                phonons=phonons, method="qhgk", storage="numpy"
            ).conductivity.sum(axis=0)
            print(
                "Conductivity from QHGK (W/m-K): %.3f"
                % (np.mean(np.diag(qhgk_cond_matrix)))
            )
            print(qhgk_cond_matrix)


if __name__ == "__main__":
    main()
