"""
This script computes anharmonic properties and thermal conductivity
for silicon bulk using hiPhive and the Boltzmann Transport Equation (BTE).

Usage:
    python TC.py <c_parameter>
"""

import sys
import numpy as np
import pickle
from kaldo.conductivity import Conductivity
from kaldo.forceconstants import ForceConstants
from kaldo.phonons import Phonons


def main():
    """
    Main function to compute and process phonon thermal conductivity.

    Reads a scaling factor from the command line and processes different methods,
    k-point meshes, and temperatures.
    """
    # Scaling factor from command line
    c = float(sys.argv[1])
    ini_sc = [7, 7, 7]
    # Directory for DFT-derived force constants
    root = "../../0_DFT/" + "c_{}/fc_kaldo_sc{}/".format(c, ini_sc[0])
    method_list = ["corr_", "gauss_sigma1_", "gauss_sigma0.75_", "gauss_sigma1.25_"]

    for method in method_list:
        # Configure different supercells for the second and third order force constants
        forceconstants = ForceConstants.from_folder(
            root,
            supercell=ini_sc,
            third_supercell=np.array([5, 5, 5]),
            format="shengbte-d3q",
            only_second=True,
        )
        # Loop over k-points and temperatures
        for k_points in [12]:
            for temperature in [300]:
                phonons_config = {
                    "kpts": [k_points, k_points, k_points],
                    "is_classic": False,
                    "temperature": temperature,  # simulation temperature in Kelvin
                    "folder": "ald_{}".format(method),
                    "storage": "numpy",
                }
                # Set up phonon object with the configuration and force constants
                phonons = Phonons(forceconstants=forceconstants, **phonons_config)
                print("masses: ", phonons.atoms.get_masses())

                # Load bandwidth data and apply spatial corrections
                _ = phonons.heat_capacity  # This call creates necessary folders.
                root_band = "ald_c_{}_sc7/{}_{}_{}/{}/quantum/bandwidth.npy".format(
                    c, k_points, k_points, k_points, temperature
                )
                root_tmp = "./ald_{}/{}_{}_{}/{}/quantum/bandwidth.npy".format(
                    method, k_points, k_points, k_points, temperature
                )
                pwd = "../"
                bw = np.load(pwd + root_band)
                with open("../../bw/spatial_{}.pkl".format(method), "rb") as f:
                    spl = pickle.load(f)
                bw_tot = bw + spl(phonons.frequency.flatten()).reshape(bw.shape)
                np.save(root_tmp, bw_tot)
                print("\n")

                # Compute and print thermal conductivity using RTA method
                rta_cond_matrix = Conductivity(
                    phonons=phonons, method="rta", storage="numpy"
                ).conductivity.sum(axis=0)
                print(
                    "Conductivity from RTA (W/m-K): %.3f"
                    % (np.mean(np.diag(rta_cond_matrix)))
                )
                print(rta_cond_matrix)
                print("\n")

                # Compute and print thermal conductivity using QHGK method
                qhgk_cond_matrix = Conductivity(
                    phonons=phonons, method="qhgk", storage="numpy"
                ).conductivity.sum(axis=0)
                print(
                    "Conductivity from QHGK (W/m-K): %.3f"
                    % (np.mean(np.diag(qhgk_cond_matrix)))
                )
                print(qhgk_cond_matrix)


if __name__ == "__main__":
    main()
