"""
Minimal script to compute anharmonic properties and thermal conductivity
for silicon bulk using hiPhive, based on force constants.
"""

import sys
import pickle
import numpy as np
from kaldo.conductivity import Conductivity
from kaldo.forceconstants import ForceConstants
from kaldo.phonons import Phonons


def g_factor(c, m1, m2):
    """
    Compute the isotopic scattering factor and average mass.

    Parameters:
        c (float): Concentration.
        m1 (float): Mass for component 1.
        m2 (float): Mass for component 2.

    Returns:
        tuple: (g, m_avg) where g is the scattering factor and m_avg is the average
        mass.
    """
    m_avg = c * m1 + (1 - c) * m2
    g = c * (1 - m1 / m_avg) ** 2 + (1 - c) * (1 - m2 / m_avg) ** 2
    return g, m_avg


def g_factor_iso(c, m1_iso, m2_iso):
    """
    Compute the isotopic scattering factor with isotope masses dictionaries.

    Parameters:
        c (float): Concentration.
        m1_iso (dict): Isotope masses with their relative abundances for component 1.
        m2_iso (dict): Isotope masses with their relative abundances for component 2.

    Returns:
        tuple: (g, m_avg) where g is the scattering factor and m_avg is the average
        mass.
    """
    m1 = np.array([m * m1_iso[m] for m in m1_iso.keys()]).sum()
    m2 = np.array([m * m2_iso[m] for m in m2_iso.keys()]).sum()
    m_avg = c * m1 + (1 - c) * m2
    g = 0
    for m in m1_iso.keys():
        g += c * m1_iso[m] * (1 - m / m_avg) ** 2
    for m in m2_iso.keys():
        g += (1 - c) * m2_iso[m] * (1 - m / m_avg) ** 2
    return g, m_avg


# Isotope abundances for Ge and Si
ge_iso = {
    70: 20.5 / 100,
    72: 27.4 / 100,
    73: 7.76 / 100,
    74: 36.5 / 100,
    76: 7.75 / 100,
}

si_iso = {
    28: 92.2 / 100,
    29: 4.7 / 100,
    30: 3.1 / 100,
}

Ge_mass = 72.63
Si_mass = 28.086

# Get concentration from command line arguments
c = float(sys.argv[1])

# Define initial supercell dimensions
ini_sc = [7, 7, 7]
root = "../../0_DFT/" + "c_{}/fc_kaldo_sc{}/".format(c, ini_sc[0])

# Get force constants with different supercells for second and third order
forceconstants = ForceConstants.from_folder(
    root,
    supercell=ini_sc,
    third_supercell=np.array([5, 5, 5]),
    format="shengbte-d3q",
    only_second=True,
)

# Loop over different k-point meshes and temperatures
for k_points in [12]:
    for temperature in [300]:
        # Phonon configuration parameters
        phonons_config = {
            "kpts": [k_points, k_points, k_points],
            "is_classic": False,
            "temperature": temperature,
            "folder": "ald_c_{}_sc{}".format(c, ini_sc[0]),
            "storage": "numpy",
        }
        # Set up phonon object with the configuration and computed force constants
        phonons = Phonons(forceconstants=forceconstants, **phonons_config)
        print("masses: ", phonons.atoms.get_masses())

        # Compute phonon properties: frequency and heat capacity (creates folders)
        freq = phonons.frequency
        hc = phonons.heat_capacity

        # Load bandwidth data and apply isotopic correction
        root_bw = "ald_c_{}_sc7/{}_{}_{}/{}/quantum/bandwidth.npy".format(
            c, k_points, k_points, k_points, temperature
        )
        pwd = "../"
        bw = np.load(pwd + root_bw)
        with open("../../bw/disorder_c{}.pkl".format(c), "rb") as f:
            spl = pickle.load(f)

        iso_correction = (
            g_factor_iso(c, m1_iso=ge_iso, m2_iso=si_iso)[0]
            / g_factor(c, m1=Ge_mass, m2=Si_mass)[0]
        )
        print("iso correction=", iso_correction)
        bw_tot = bw + iso_correction * spl(phonons.frequency.flatten()).reshape(
            bw.shape
        )
        np.save(root_bw, bw_tot)
        print("\n")

        # Compute thermal conductivity using the RTA method
        rta_cond_matrix = Conductivity(
            phonons=phonons, method="rta", storage="numpy"
        ).conductivity.sum(axis=0)
        print(
            "Conductivity from RTA (W/m-K): %.3f" % (np.mean(np.diag(rta_cond_matrix)))
        )
        print(rta_cond_matrix)
